Unit ALD1R_p;
interface
uses
SysUtils, Math, { Delphi }
Lstruct, Lfunc, ALN1R_p, ALN1R4_p;

procedure ALD1R(var G :Array of Real; MDG :Integer; M :Integer;
                N :Integer; var H :Array of Real;
                var X :Array of Real; var XNORM :Real;
                var W :Array of Real; var INDEX :Array of Integer;
                var IERR :Integer);

implementation

procedure ALD1R(var G :Array of Real; MDG :Integer; M :Integer;
                N :Integer; var H :Array of Real;
                var X :Array of Real; var XNORM :Real;
                var W :Array of Real; var INDEX :Array of Integer;
                var IERR :Integer);
var
J,IW,I,NP1,IZ,IY,IWDUAL,IF_ :Integer;
RNORM,FAC :Real;
const
ZERO :Real = 0.0E0;
ONE :Real = 1.0E0;
label
_10,_20,_30,_40,_50,_60,_70,_80,_90,_100,_110,_120,_130;
begin

{ ******************************************************* }

{     PURPOSE: **** LEAST DISTANCE PROGRAMMING ********** }

{   GIVEN AN M BY N  MATRIX  .G. AND  AN  M-VECTOR .H. }
{   COMPUTE  AN  N-VECTOR .X.  WHICH SOLVES THE LEAST }
{   SQUARES PROBLEM: }
{           G * X >= H     SUBJECT TO //X// - MIN }

{   SOURCE:LAWSON C.L.,HANSON R.J. 'SOLVING LEAST SQUARES }
{   PROBLEM' , PRENTICE-HALL INC.,ENGLEWOOE CLIFFS,NEW }
{   JERSEY, 1974. }

{     PARAMETERS: }
{  G,MDG,M,N - MDG IS THE FIRST DIMENSIONING PARAMETER FOR }
{              THE ARRAY .G( ).  ON ENTRY G( ) CONTAINS THE }
{              M BY N MATRIX .G. }
{  H         - ON ENTRY H CONTAINS THE M-VECTOR H. }
{  X         - ON ENTRY X NEED NOT BE INITIALIZED. }
{              ON EXIT X WILL CONTAIN THE SOLUTION VECTOR }
{              (IF IERR=0). }
{  XNORM     - ON EXIT RNORM CONTAINS THE EUCLIEEAN NORM OF }
{              THE SOLUTION VECTOR X. }
{  W( )      - AN (N+1)*(M+2)+2*M - ARRAY OF WORKING SPACE. }
{  INDEX( )  - AN INTEGER WORKING ARRAY OF LEANGTH AT }
{              LEAST M. }
{  IERR      - THIS IS A DIAGNOSTICAL PARAMETER USED AS A }
{              SUCCESS-FAILURE FLAG WITH  MEANINGS: }
{               0 - THE SOLUTION HAS BEEN COMPUTEE }
{                   SUCCESSFULLY. }
{               1 - THE SYSTEM HAS NO SOLUTION. }
{              65 - THE DIMENSIONS OF THE PROBLEM ARE BAD. }
{   EXTERNAL SUBROUTINES: }
{            ALN1R  (NONNEGATIVE LIST SQUARES), }
{            ALN1R4 (COMPUTE DIFFERENCE DIFF=X-Y) }

{ ********************************************************* }

if ( N <= 0 ) 
 then goto _120;
for J:=1 to N do
 begin
_10:
  X[J-1] := ZERO;
 end;
XNORM := ZERO;
if ( M <= 0 ) 
 then goto _110;

{     THE DECLAREE DIMENSION OF W MUST BE AT LEAST }
{                (N+1)*(M+2)+2*M }

{    FIRST (N+1)*M LOCKS OF W = MATRIX E FOR PROBLEM NNLS }
{     NEXT     N+1 LOCKS OF W = VECTOR F FOR PROBLEM NNLS }
{     NEXT     N+1 LOCKS OF W = VECTOR Z FOR PROBLEM NNLS }
{     NEXT       M LOCKS OF W = VECTOR Y FOR PROBLEM NNLS }
{     NEXT       M LOCKS OF W = VECTOR WDUAL FOR NNLS }
{      COPY G**T INTO FIRST N ROWS AND M COLUMNS OF E }
{      COPY H**T INTO ROW N+1 OF E }

IW := 0;
for J:=1 to M do
 begin
  for I:=1 to N do
   begin
    IW := IW+1;
_20:
    W[IW-1] := G[(J-1)+(I-1)*MDG];
   end;
  IW := IW+1;
_30:
  W[IW-1] := H[J-1];
 end;
IF_ := IW+1;
{      STORE N ZEROS FOLLOWEE BY A ONE INTO F }
for I:=1 to N do
 begin
  IW := IW+1;
  W[IW-1] := ZERO;
_40:
 end;
W[IW+0] := ONE;

NP1 := N+1;
IZ := IW+2;
IY := IZ+NP1;
IWDUAL := IY+M;

ALN1R(W,NP1,NP1,M,W[IF_-1],W[IY-1],RNORM,W[IWDUAL-1],
     W[IZ-1],INDEX,IERR);
{      USE THE FOLLOWING RETURN IF UNSUCCESSFUL IN NNLS }
if ( IERR <> 0 ) 
 then exit;
if ( RNORM ) < 0
 then goto _130
 else if ( RNORM ) > 0
       then goto _50
       else goto _130;
_50:
FAC := ONE;
IW := IY-1;
for I:=1 to M do
 begin
  IW := IW+1;
{      HERE WE ARE USING THE SOLUTION VECTOR Y }
  FAC := FAC-H[I-1]*W[IW-1];
_60:
 end;

if ( ALN1R4(ONE+FAC,ONE) ) < 0
 then goto _130
 else if ( ALN1R4(ONE+FAC,ONE) ) > 0
       then goto _70
       else goto _130;
_70:
FAC := ONE/FAC;
for J:=1 to N do
 begin
  IW := IY-1;
  for I:=1 to M do
   begin
    IW := IW+1;
{      HERE WE ARE USING THE SOLUTION VECTOR Y }
    X[J-1] := X[J-1]+G[(I-1)+(J-1)*MDG]*W[IW-1];
_80:
   end;
  X[J-1] := X[J-1]*FAC;
_90:
 end;
for J:=1 to N do
 begin
  XNORM := XNORM+IntPower(X[J-1],2);
_100:
 end;
XNORM := Sqrt(XNORM);
{      SUCCESSFUL RETURN }
_110:
IERR := 0;
exit;
{      ERROR RETURN      N.LE.0 }
_120:
IERR := 65;
exit;
{      RETURNING WITH CONSTRAINS NOT COMPATIBLE }
_130:
IERR := 1;
end;

end.
